"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createComputeBudgetIx = exports.sendTransaction = void 0;
const nodewallet_1 = __importDefault(require("@project-serum/anchor/dist/cjs/nodewallet"));
const web3_js_1 = require("@solana/web3.js");
async function sendTransaction(provider, ixs, alts, opts = {}) {
    var _a, _b, _c, _d;
    const connection = provider.connection;
    const latestBlockhash = await connection.getLatestBlockhash((_b = (_a = opts.preflightCommitment) !== null && _a !== void 0 ? _a : provider.opts.preflightCommitment) !== null && _b !== void 0 ? _b : 'finalized');
    const payer = provider.wallet;
    if (opts.prioritizationFee) {
        ixs = [(0, exports.createComputeBudgetIx)(opts.prioritizationFee), ...ixs];
    }
    const message = web3_js_1.MessageV0.compile({
        payerKey: provider.wallet.publicKey,
        instructions: ixs,
        recentBlockhash: latestBlockhash.blockhash,
        addressLookupTableAccounts: alts,
    });
    let vtx = new web3_js_1.VersionedTransaction(message);
    if ((_c = opts === null || opts === void 0 ? void 0 : opts.additionalSigners) === null || _c === void 0 ? void 0 : _c.length) {
        vtx.sign([...opts === null || opts === void 0 ? void 0 : opts.additionalSigners]);
    }
    if (typeof payer.signTransaction === 'function' &&
        !(payer instanceof nodewallet_1.default)) {
        vtx = (await payer.signTransaction(vtx));
    }
    else {
        // Maybe this path is only correct for NodeWallet?
        vtx.sign([payer.payer]);
    }
    const signature = await connection.sendRawTransaction(vtx.serialize(), {
        skipPreflight: true, // mergedOpts.skipPreflight,
    });
    // const signature = await connection.sendTransactionss(
    //   vtx as any as VersionedTransaction,
    //   {
    //     skipPreflight: true,
    //   },
    // );
    if (opts.postSendTxCallback) {
        try {
            opts.postSendTxCallback({ txid: signature });
        }
        catch (e) {
            console.warn(`postSendTxCallback error ${e}`);
        }
    }
    const txConfirmationCommitment = (_d = opts.txConfirmationCommitment) !== null && _d !== void 0 ? _d : 'processed';
    let status;
    if (latestBlockhash.blockhash != null &&
        latestBlockhash.lastValidBlockHeight != null) {
        status = (await connection.confirmTransaction({
            signature: signature,
            blockhash: latestBlockhash.blockhash,
            lastValidBlockHeight: latestBlockhash.lastValidBlockHeight,
        }, txConfirmationCommitment)).value;
    }
    else {
        status = (await connection.confirmTransaction(signature, txConfirmationCommitment)).value;
    }
    if (status.err) {
        console.warn('Tx status: ', status);
        throw new MangoError({
            txid: signature,
            message: `${JSON.stringify(status)}`,
        });
    }
    return signature;
}
exports.sendTransaction = sendTransaction;
const createComputeBudgetIx = (microLamports) => {
    const computeBudgetIx = web3_js_1.ComputeBudgetProgram.setComputeUnitPrice({
        microLamports,
    });
    return computeBudgetIx;
};
exports.createComputeBudgetIx = createComputeBudgetIx;
class MangoError extends Error {
    constructor({ txid, message }) {
        super();
        this.message = message;
        this.txid = txid;
    }
}
//# sourceMappingURL=rpc.js.map